package de.hdm_stuttgart.mi.sudoku.ui;

import javax.servlet.annotation.WebServlet;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Title;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.ui.AbstractComponent;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;

import de.hdm_stuttgart.mi.sudoku.model.IntegerConverter;
import de.hdm_stuttgart.mi.sudoku.model.SudokuBoard;

/**
 * Playing Sudoku. Inspired by http://www.sudoku-solutions.com/
 *
 */
@Title("Sudoku")
@Theme("sudoku") // In favour of @Theme("valo"), see https://vaadin.com/forum#!/thread/9359396
public class HelloUi  extends UI {

	private static final long serialVersionUID = -307591917714387247L;

	@Override
	protected void init(VaadinRequest request) {

		final VerticalLayout mainLayout = new VerticalLayout();
		setContent(mainLayout);   // Attach to the UI

		final Table table = new Table("Sudoku");
		mainLayout.addComponent(table);

		
		table.setCellStyleGenerator(new Table.CellStyleGenerator() {
			private static final long serialVersionUID = 2901054012453350566L;

			@Override
			public String getStyle(Table source, Object itemId, Object propertyId) {
				if (propertyId instanceof Integer && itemId instanceof Integer) {
					final int col = (Integer) propertyId;
					final int row = (Integer) itemId;

					final StringBuffer format = new StringBuffer();

					switch(row ) {
					case 2:
					case 5:
						format.append("bottom");
						break;
					}

					switch(col) {
					case 2:
					case 5:
						format.append("right");
						break;
					} 
					if (0 < format.length()) {
						return format.toString();
					}
				}
				return null;
			}
		});

		table.setColumnHeaderMode(Table.ColumnHeaderMode.HIDDEN);

		table.setEditable(true); 
		table.setImmediate(true);
		for (int y = 0; y < SudokuBoard.size; y++) {
			table.addContainerProperty (y, AbstractComponent.class, null);
		}

		final SudokuBoard board = new SudokuBoard();

		for (int y = 0; y < SudokuBoard.size; y++) {
			final AbstractComponent tf[] = new AbstractComponent[SudokuBoard.size];
			for (int x = 0; x < SudokuBoard.size; x++) {
				final TextField t = new TextField();
				tf[x] = t;

				t.setWidth("4ex");
				t.setConversionError("{1}"); // See https://vaadin.com/forum/#!/thread/3625631
				t.setConverter(new IntegerConverter(board, y, x));
				t.setImmediate(true);
				t.setPropertyDataSource(board.cells[y][x]);
			}

			table.addItem(tf, new Integer(y));
		}
	}

	/**  Deployment description .
	 *
	 */
	@WebServlet(urlPatterns = "/*")
	@VaadinServletConfiguration(ui = HelloUi.class, productionMode = false)
	public static class MyUIServlet extends VaadinServlet {
		private static final long serialVersionUID = -2771410633323765889L;
	}
}
